#include <iostream>
#include <fstream>
#include <boost/dynamic_bitset.hpp>
#include <boost/lexical_cast.hpp>
#include <string>
#include <vector>

int main(int argc, char** argv)
{

	if(argc != 4)
	{
		std::cerr << "Usage : " << argv[0] << " alignments_file scaffolds_file genome_length\n";
		return 1;
	}

	std::ifstream alignments_file(argv[1]);
	std::ifstream scaffolds_file(argv[2]);
	const size_t dnalength = boost::lexical_cast<size_t>(argv[3]);

	boost::dynamic_bitset<> bits(dnalength+1); // +1 because swift starts indexing from 1

	size_t scaffold_corrected_lengths = 0;
	size_t scaffold_ordered_coverage_lengths = 0;

	if(!alignments_file)
		std::cerr << "Could not open alignments file " << argv[1] << '\n';

	std::vector<size_t> scaffold_alignment_lengths;

	std::string previous_alignment_scaffold;

	while(alignments_file)
	{
		std::string alignment_scaffold;
		getline(alignments_file, alignment_scaffold, ';');
		if(previous_alignment_scaffold == alignment_scaffold) // For coverages, only take into account the first (best) alignment
		{
			alignments_file.ignore(INT_MAX, '\n');
			continue;
		}
		previous_alignment_scaffold = alignment_scaffold;

		for(int i = 0; i < 2; ++i )
			alignments_file.ignore(4096, ';');
		if(!alignments_file)
			break;

		size_t ordered_coverage_length;
		alignments_file >> ordered_coverage_length;
		scaffold_ordered_coverage_lengths += ordered_coverage_length;
		scaffold_alignment_lengths.push_back(ordered_coverage_length);
		alignments_file.ignore(4096, ';');

		ordered_coverage_length = 0;

		scaffolds_file.ignore(INT_MAX, '\n');
		while(scaffolds_file && scaffolds_file.peek() != '>')
		{
			std::string temp;
			getline(scaffolds_file, temp);
			ordered_coverage_length = temp.length();
			for(size_t i = 0; i < temp.length(); ++i)
				if(temp[i] == 'N')
					--ordered_coverage_length;
			scaffold_corrected_lengths += ordered_coverage_length;
		}

		while(alignments_file)
		{
			for(int i = 0 ; i < 2 ; ++i)
				alignments_file.ignore(4096, ',');
			unsigned int startPos, endPos;

			alignments_file >> startPos;
			alignments_file.ignore(4096, ',');
			alignments_file >> endPos;

			if(startPos > endPos)
			{
				unsigned int temp = startPos;
				startPos=endPos;
				endPos = temp;
			}
			for(unsigned int i = startPos; i <= endPos; ++i)
				bits.set(i);
			alignments_file.ignore(4096, ';');
			if(alignments_file.peek() == '\n')
			{
				alignments_file.ignore(4096, '\n');
				break;
			}
		}
	}

	long double temp = (long double) bits.count() / dnalength;

	std::cout << "Genome Coverage   : " << temp;
	temp = (long double) scaffold_ordered_coverage_lengths / scaffold_corrected_lengths;
	std::cout << "\nScaffold Coverage : " << temp << '\n';


	return 0;
}
