#include <cassert>
#include <fstream>
#include <string>
#include <iostream>
#include <vector>
#include <stdint.h>
#include <climits>
#include <algorithm>
#include <boost/dynamic_bitset.hpp>

void fetch_scaffold(const std::string& name, std::istream& scaffolds_file, std::string& scaf_seq)
{
	std::string scaf_name;
	while(scaffolds_file && scaf_name != name)
	{
		while(scaffolds_file && scaffolds_file.peek() != '>')
			scaffolds_file.ignore(INT_MAX, '\n');
		getline(scaffolds_file, scaf_name);
		scaf_name.erase(0,1); // Get rid of '>'
	}

	while(scaffolds_file && scaffolds_file.peek() != '>')
	{
		std::string temp;
		getline(scaffolds_file, temp);
		scaf_seq += temp;
	}

}

int main(int argc, char** argv)
{
	if(argc < 3)
	{
		std::cerr << "Usage : " << argv[0] << " alignments_file scaffolds_file [genome_length]\n";
		return -1;
	}

	std::ifstream alignments_file(argv[1]), scaffolds_file(argv[2]);
        
        int genome_length = 0;
        if (argc>3)
            genome_length = atoi(argv[3]);

	if(!alignments_file || !scaffolds_file)
	{
		std::cerr << "Error opening input files.\n";
		return -1;
	}


	std::string curr_scaf_name;
	std::string curr_scaf_seq;

	std::vector<uint64_t> part_lengths;
	uint64_t total_part_lengths = 0;
	boost::dynamic_bitset<> positions_covered(1);

	while(true)
	{
		std::string scaf_name;
		if(!getline(alignments_file, scaf_name, ';'))
			break;
		if(scaf_name != curr_scaf_name)
		{
			curr_scaf_seq = "";
			fetch_scaffold(scaf_name, scaffolds_file, curr_scaf_seq);
			if(!scaffolds_file)
				break;
			curr_scaf_name = scaf_name;
			positions_covered.resize(curr_scaf_seq.length()+1);
			positions_covered.reset();
		}
		for(int i = 0; i < 3; ++i)
			alignments_file.ignore(INT_MAX,';');


		uint64_t curr_part_length = 0;
		while(alignments_file.peek() != '\n')
		{
			uint64_t scaf_start, scaf_end;
			alignments_file >> scaf_start;
			alignments_file.ignore(1,',');
			alignments_file >> scaf_end;
			alignments_file.ignore(INT_MAX,';');
			assert(alignments_file);
			--scaf_start; --scaf_end; // Swift indexing starts at 1 instead of 0
			if(scaf_start > scaf_end)
				std::swap(scaf_start, scaf_end);

			for(;scaf_start <= scaf_end; ++scaf_start)
				if(!positions_covered[scaf_start] && curr_scaf_seq[scaf_start] != 'N')
				{
					++curr_part_length;
					if(scaf_start < positions_covered.size())
					positions_covered.set(scaf_start);
				}
		}
		part_lengths.push_back(curr_part_length);
		total_part_lengths += curr_part_length;
		alignments_file.ignore(1,'\n');
	}

	std::sort(part_lengths.begin(), part_lengths.end());
        if (genome_length == 0) // genome length not given -> use overal scaffold length
           genome_length = total_part_lengths;

	uint64_t parts_N50 = 0;
	for(std::vector<uint64_t>::reverse_iterator i = part_lengths.rbegin();
			i != part_lengths.rend();
			++i)
	{
		parts_N50 += *i;
		if(parts_N50 >= genome_length / 2)
		{
			parts_N50 = *i;
			break;
		}
	}

	std::cout << "Normalized N50 : " << parts_N50 << '\n';

	return 0;
}
