#include <string>
#include <fstream>
#include <list>
#include <vector>
#include <boost/algorithm/string.hpp>
#include <boost/lexical_cast.hpp>
#include <algorithm>

#include "tuple_utils.h"

// compares tuples based on scaffold right edge position
bool compareTuple (const LocationTuple& t1, const LocationTuple& t2) {
	return t1.rs < t2.rs;
}

bool compareTupleReverse (const LocationTuple& t1, const LocationTuple& t2) {
	return t1.ls < t2.ls;
}

	bool operator<(const LocationTuple& t1, const LocationTuple& t2) {
		if(t1.rs != t2.rs)
			return t1.rs < t2.rs;
		if(t1.ls != t2.ls)
			return t1.ls < t2.ls;
		if(t1.re != t2.re)
			return t1.re < t2.re;
		return t1.le < t2.le;
	}

Est read_tuples(std::vector<LocationTuple>& tuples, std::istream& stream)
{
	Est est;
	stream >> est.name >> est.index >> est.length;
	long tuple_index = 0;
	while(stream && stream.peek() != '\n')
	{
		LocationTuple temp;
		stream >> temp.le >> temp.re >> temp.ls >> temp.rs;
		temp.index = tuple_index++;
		temp.esti = est.index;
		tuples.push_back(temp);
	}
	return est;
}

bool is_reverse(const LocationTuple& tuple)
{
	return tuple.re < tuple.le;
}

void write_tuples(const Est& est, const std::vector<LocationTuple>& tuples, std::ostream& stream)
{
	stream << est.name << '\t' << est.index << '\t' << est.length;
	for(auto i = tuples.begin(); i != tuples.end(); ++i)
		stream << '\t' 
			<< i->le << '\t' 
			<< i->re << '\t' 
			<< i->ls << '\t' 
			<< i->rs;
	stream << '\n';
}

void write_results(const Est& est, std::ostream& stream)
{
	stream << est.name << ';' << est.length << ';' << est.cov << ';' << est.covOrder << ';';
	for(auto i = est.alignment.begin(); i != est.alignment.end(); ++i)
		stream	<< i->le << ',' 
			<< i->re << ',' 
			<< i->ls << ',' 
			<< i->rs << ';';
	stream << '\n';
}

// reads ests from the given file, returns number of ests read
unsigned long readEsts(std::string filename, std::list<Est>& ests) {
	std::ifstream estl;
	estl.open(filename.c_str());
	if (!estl.is_open())
		return 0;
	std::string line;
	std::vector<std::string> strs;
	long esti = 1;
	while ( estl.good() ) {
		getline(estl,line);
		if ( line.length() == 0 ) break;
		boost::split(strs, line, boost::is_any_of(";"));
		if ( strs[1].length() > 0 ) {
			Est e;
			e.name = strs[0];
			e.index = esti;
			e.length = boost::lexical_cast< long >(strs[1]);
			ests.push_back(e);
			esti++;
		}
	}
	return ests.size();
}

