#pragma once

#include <list>
#include <string>
#include <vector>
#include <iosfwd>

struct LocationTuple {
  LocationTuple() {};
  LocationTuple(const LocationTuple &copy) : index(copy.index), 
    esti(copy.esti), le(copy.le), re(copy.re), ls(copy.ls), rs(copy.rs), tag(copy.tag) {};
  long index; // used for tracing the coverage path
  long esti; // Est index
  long le; // Est left edge position
  long re; // Est right edge position
  long ls; // Scaffold left edge position
  long rs; // Scaffold right edge position
  int tag; // extra information used when limiting gap
  bool operator==(const LocationTuple& tuple) const { return ( le == tuple.le && re == tuple.re && ls == tuple.ls && rs == tuple.rs); } 
  bool operator!=(const LocationTuple& tuple) const { return !(*this == tuple); }
};

struct Est {
  Est() {cov=0; covOrder=0;}
  std::string name; // Est name in the data
  std::list<LocationTuple> alignment; // list of tuples giving the alignment
  long index;
  long length;
  long cov; // number of covered bases
  long covOrder; // number of covered bases with ordering taken into account
};

bool compareTuple (const LocationTuple& t1, const LocationTuple& t2);
bool compareTupleReverse (const LocationTuple& t1, const LocationTuple& t2);
unsigned long readEsts(std::string, std::list<Est>&);
void write_tuples(const Est& est, const std::vector<LocationTuple>& tuples, std::ostream& stream);
Est read_tuples(std::vector<LocationTuple>& tuples, std::istream& stream);
void write_tuples_uniq(const Est& est, std::vector<LocationTuple>& tuples, std::ostream& stream);
void write_results(const Est& est, std::ostream& stream);
bool operator<(const LocationTuple& t1, const LocationTuple& t2);
