// blasttablereporter.hpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
//
// Additions / changes by Peter Husemann
//

#ifndef _SWIFT_BLASTTABLEREPORTER_
#define _SWIFT_BLASTTABLEREPORTER_

#include "alignment.hpp"
#include "reporter.hpp"
#include <boost/shared_ptr.hpp>
#include <iosfwd>
#include <vector>

namespace swift
  {


  class KarlinAltschul;
  class SequenceDatabase;


  class BlastTableReporter : public Reporter
    {
    public:
      BlastTableReporter(
        boost::shared_ptr<SequenceDatabase const> queries,
        boost::shared_ptr<SequenceDatabase const> subjects,
        boost::shared_ptr<KarlinAltschul const> kaStats,
        std::ostream & outStream
      );
      ~BlastTableReporter();

      virtual void append(
        Alignment * alignment,
        unsigned int queryId,
        unsigned int subjectId
      );
      virtual void report(bool &compassemboutput);
      virtual unsigned int size() const;
      virtual bool isEmpty() const;
      virtual void clear();

      void message(const char*) const;


    private:
      BlastTableReporter();

      void removeDuplicates();
      void formatEValue(double eValue, char * buffer) const;
      void formatBitScore(double bitScore, char * buffer) const;

    public:
      struct SubjectAlignment
        {
          SubjectAlignment(Alignment * a, unsigned int sid);

          Alignment * alignment;
          unsigned int subjectId;
        };

    private:
      typedef std::vector<SubjectAlignment> Alignments;
      typedef Alignments::iterator AIterator;

      unsigned int queryId_;
      Alignments alignments_;
      std::ostream & outStream_;
      boost::shared_ptr<SequenceDatabase const> queries_;
      boost::shared_ptr<SequenceDatabase const> subjects_;
      boost::shared_ptr<KarlinAltschul const> kaStats_;
    };


  inline
  bool BlastTableReporter::isEmpty() const
    {
      return alignments_.empty();
    }


  inline
  unsigned int BlastTableReporter::size() const
    {
      return alignments_.size();
    }


  inline
  BlastTableReporter::SubjectAlignment::SubjectAlignment(
    Alignment * a,
    unsigned int sid
  ) :
      alignment(a),
      subjectId(sid)
  {}


} // namespace swift

#endif

