// karlinaltschul.cpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

#include "karlinaltschul.hpp"

namespace swift
{


KarlinAltschul::KarlinAltschulParameters const
KarlinAltschul::KaParameters[][10] = {
    {
        { 1.1000 , 0.3330 , 0.5490 },   /*  +1/-1 */
        { 0.2640 , 0.0532 , 0.0722 },   /*  +2/-1 */
        { 0.0000 , 0.0000 , 0.0000 },
        { 0.0000 , 0.0000 , 0.0000 },
        { 0.0000 , 0.0000 , 0.0000 },
        { 0.0000 , 0.0000 , 0.0000 },
        { 0.0000 , 0.0000 , 0.0000 },
        { 0.0000 , 0.0000 , 0.0000 },
        { 0.0000 , 0.0000 , 0.0000 },
        { 0.0000 , 0.0000 , 0.0000 }
    },  
    {
        { 1.3300 , 0.6210 , 1.1200 },   /*  +1/-2 */
        { 0.5490 , 0.3330 , 0.5490 },   /*  +2/-2 */
        { 0.2710 , 0.1300 , 0.2220 },   /*  +3/-2 */
        { 0.1320 , 0.0532 , 0.0722 },   /*  +4/-2 */
        { 0.0514 , 0.0250 , 0.0135 },   /*  +5/-2 */
        { 0.0000 , 0.0000 , 0.0000 },
        { 0.0000 , 0.0000 , 0.0000 },
        { 0.0000 , 0.0000 , 0.0000 },
        { 0.0000 , 0.0000 , 0.0000 },
        { 0.0000 , 0.0000 , 0.0000 }
    },  
    {
        { 1.3700 , 0.7110 , 1.3100 },   /*  +1/-3 */
        { 0.6340 , 0.4080 , 0.9120 },   /*  +2/-3 */
        { 0.3660 , 0.3330 , 0.5490 },   /*  +3/-3 */
        { 0.2270 , 0.1610 , 0.3060 },   /*  +4/-3 */
        { 0.1440 , 0.0952 , 0.0158 },   /*  +5/-3 */
        { 0.0882 , 0.0532 , 0.0722 },   /*  +6/-3 */
        { 0.0494 , 0.0291 , 0.0264 },   /*  +7/-3 */
        { 0.0212 , 0.0268 , 0.0055 },   /*  +8/-3 */
        { 0.0000 , 0.0000 , 0.0000 },
        { 0.0000 , 0.0000 , 0.0000 }
    },  
    {
        { 1.3800 , 0.7380 , 1.3600 },   /*  +1/-4 */
        { 0.6660 , 0.6210 , 1.1200 },   /*  +2/-4 */
        { 0.4100 , 0.3400 , 0.8110 },   /*  +3/-4 */
        { 0.2750 , 0.3330 , 0.5490 },   /*  +4/-4 */
        { 0.1920 , 0.1760 , 0.3570 },   /*  +5/-4 */
        { 0.1360 , 1.6300 , 0.2220 },   /*  +6/-4 */
        { 0.0957 , 0.0809 , 0.1320 },   /*  +7/-4 */
        { 0.0661 , 0.0532 , 0.0722 },   /*  +8/-4 */
        { 0.0435 , 0.0327 , 0.0350 },   /*  +9/-4 */
        { 0.0257 , 7.9300 , 0.0135 }    /* +10/-4 */
    },  
    {
        { 1.3900 , 0.7470 , 1.3800 },   /*  +1/-5 */
        { 0.6810 , 0.5150 , 1.2400 },   /*  +2/-5 */
        { 0.4320 , 0.3960 , 0.9970 },   /*  +3/-5 */
        { 0.3010 , 0.3060 , 0.7530 },   /*  +4/-5 */
        { 0.2200 , 0.3330 , 0.5490 },   /*  +5/-5 */
        { 0.1640 , 0.1840 , 0.3900 },   /*  +6/-5 */
        { 0.1240 , 0.1400 , 0.2700 },   /*  +7/-5 */
        { 0.0945 , 0.1030 , 0.1820 },   /*  +8/-5 */
        { 0.0713 , 0.0733 , 0.1180 },   /*  +9/-5 */
        { 0.0529 , 0.0532 , 0.0722 }    /* +10/-5 */
    },  
    {
        { 1.3900 , 0.7490 , 1.3800 },   /*  +1/-6 */
        { 0.6870 , 0.7110 , 1.3100 },   /*  +2/-6 */
        { 0.4440 , 0.6210 , 1.1200 },   /*  +3/-6 */
        { 0.3170 , 1.1700 , 0.9120 },   /*  +4/-6 */
        { 0.2370 , 0.2860 , 0.7160 },   /*  +5/-6 */
        { 0.1830 , 0.3330 , 0.5490 },   /*  +6/-6 */
        { 0.1440 , 0.1890 , 0.4140 },   /*  +7/-6 */
        { 0.1140 , 1.4100 , 0.3060 },   /*  +8/-6 */
        { 0.0904 , 1.6300 , 0.2220 },   /*  +9/-6 */
        { 0.0718 , 1.7900 , 0.1580 }    /* +10/-6 */
    },  
    {
        { 1.3900 , 0.7500 , 1.3900 },   /*  +1/-7 */
        { 0.6900 , 0.5480 , 1.3400 },   /*  +2/-7 */
        { 0.4510 , 0.4550 , 1.2100 },   /*  +3/-7 */
        { 0.3270 , 0.3860 , 1.0300 },   /*  +4/-7 */
        { 0.2490 , 0.3270 , 0.8540 },   /*  +5/-7 */
        { 0.1960 , 0.2730 , 0.6900 },   /*  +6/-7 */
        { 0.1570 , 0.3330 , 0.5490 },   /*  +7/-7 */
        { 0.1270 , 0.1920 , 0.4310 },   /*  +8/-7 */
        { 0.1040 , 0.1610 , 0.3340 },   /*  +9/-7 */
        { 0.0855 , 0.1320 , 0.2560 }    /* +10/-7 */
    },  
    {
        { 1.3900 , 0.7500 , 1.3900 },   /*  +1/-8 */
        { 0.6920 , 0.7380 , 1.3600 },   /*  +2/-8 */
        { 0.4550 , 0.4720 , 1.2700 },   /*  +3/-8 */
        { 0.3330 , 0.6210 , 1.1200 },   /*  +4/-8 */
        { 0.2570 , 0.3570 , 0.9650 },   /*  +5/-8 */
        { 0.2050 , 1.1000 , 0.8110 },   /*  +6/-8 */
        { 0.1670 , 0.2630 , 0.6710 },   /*  +7/-8 */
        { 0.1370 , 0.3330 , 0.5490 },   /*  +8/-8 */
        { 0.1140 , 0.1940 , 0.4450 },   /*  +9/-8 */
        { 0.0958 , 1.3100 , 0.3570 }    /* +10/-8 */
    },  
    {
        { 1.3900 , 0.7500 , 1.3900 },   /*  +1/-9 */
        { 0.6920 , 0.5580 , 1.3700 },   /*  +2/-9 */
        { 0.4580 , 0.7110 , 1.3100 },   /*  +3/-9 */
        { 0.3370 , 0.4270 , 1.1900 },   /*  +4/-9 */
        { 0.2630 , 0.3780 , 1.0500 },   /*  +5/-9 */
        { 0.2110 , 1.1700 , 0.9120 },   /*  +6/-9 */
        { 0.1740 , 0.2950 , 0.7790 },   /*  +7/-9 */
        { 0.1450 , 0.2550 , 0.6570 },   /*  +8/-9 */
        { 0.1220 , 0.3330 , 0.5490 },   /*  +9/-9 */
        { 0.1040 , 0.1960 , 0.4560 }    /* +10/-9 */
    },  
    {
        { 1.3900 , 0.7500 , 1.3900 },   /*  +1/-10 */
        { 0.6930 , 0.7470 , 1.3800 },   /*  +2/-10 */
        { 0.4600 , 0.4910 , 1.3300 },   /*  +3/-10 */
        { 0.3400 , 1.0600 , 1.2400 },   /*  +4/-10 */
        { 0.2670 , 0.6210 , 1.1200 },   /*  +5/-10 */
        { 0.2160 , 1.0300 , 0.9970 },   /*  +6/-10 */
        { 0.1790 , 0.3210 , 0.8710 },   /*  +7/-10 */
        { 0.1510 , 1.0700 , 0.7530 },   /*  +8/-10 */
        { 0.1280 , 0.2500 , 0.6460 },   /*  +9/-10 */
        { 0.1100 , 0.3330 , 0.5490 }    /* +10/-10 */
    }  
};


}

