// reporter.hpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
//
// Additions / changes by Peter Husemann
//

#ifndef _SWIFT_REPORTER_
#define _SWIFT_REPORTER_

namespace swift
{


class Alignment;


class Reporter
{
    public:
    virtual ~Reporter(){};

    /*!
        Adds an \a alignment to the report. The reporter takes ownership
        of the alignment. The corresponding query and subject sequences are
        identified by \a queryId and \a subjectId, respectively.
    */
    virtual void append(
        Alignment * alignment,
        unsigned int queryId = 0,
        unsigned int subjectId = 0
    ) = 0;

    /*!
        Generates the report. Afterwards the reporter is empty.

        \sa isEmpty
    */
    virtual void report(bool &compassemb) = 0;

    /*!
        Returns the number of alignments in to the reporter.
    */
    virtual unsigned int size() const = 0;

    /*!
        Returns \c true if there are no alignments in the reporter.
    */
    virtual bool isEmpty() const = 0;

    /*!
        Clears the reporter. Afterwards the reporter is empty.

        \sa isEmpty.
    */
    virtual void clear() = 0;

    protected:
    Reporter(){};
};


} // namespace swift

#endif

