// scorematrix.hpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  

#ifndef _SWIFT_SCOREMATRIX_
#define _SWIFT_SCOREMATRIX_

#include "masksensitivity.hpp"

namespace swift
{


class ScoreMatrix
{
    public:
    ScoreMatrix();

    ScoreMatrix(
        int matchScore,
        int mismatchScore,
        MaskSensitivity ms = MaskInsensitive
    );

    ScoreMatrix(ScoreMatrix const & other);

    ~ScoreMatrix();

    int const * operator[](unsigned char row) const;

    void setEntry(unsigned char row, unsigned char column, int score);

    ScoreMatrix & operator=(ScoreMatrix const & other);

    private:
    static unsigned int MatrixSize;

    int * matrix_;
};


/*!
    Returns row number \a row.
*/
inline
int const * ScoreMatrix::operator[](unsigned char row) const
{
    return matrix_ + (row << 4);
}


/*!
    Sets the entry on row \a row and column \a column to \a score.
*/
inline
void ScoreMatrix::setEntry(unsigned char row, unsigned char column, int score)
{
    *(matrix_ + (row << 4) + column) = score;
}


} // namespace swift

#endif

