// sequencedatabase.hpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// 

#ifndef _SWIFT_SEQUENCEDATABASE_
#define _SWIFT_SEQUENCEDATABASE_

#include "sequence.hpp"
#include <string>

namespace swift {

// SequenceDatabase class declaration ---------------------------------------

class SequenceDatabase
{
    public:
    typedef size_t size_type;

    /*!
        Returns the \a n th sequence, where [\a begin, \a end) defines the
        optional interest range.
    */
    virtual Sequence * sequence(
        size_type n,
        Sequence::size_type begin = 0,
        Sequence::size_type end = Sequence::npos
    ) const = 0;

    /*!
        Returns the offset for the \a n th sequence.
    */
    virtual size_type offset(size_type n) const = 0;

    /*!
        Returns the index for the sequence containing position \a position,
        or \l count() if no such sequence exists. The argument \a index
        can be used as hint to where the position may be found. It must be
        in the closed range [0, count()].
    */
    virtual size_type find(
        size_type position,
        size_type index = 0
    ) const = 0;

    /*!
        Returns the size of the \a n th sequence.
    */
    virtual Sequence::size_type size(size_type n) const = 0;

    /*!
        Returns the size of the sequence database.
    */
    virtual size_type size() const = 0;

    /*!
        Returns the number of sequences in the database.
    */
    virtual size_type count() const = 0;

    /*!
        Returns the text associated with the \a n th sequence. The
        returned string may be empty or null.
    */
    virtual std::string text(size_type n) const = 0;
};

// --------------------------------------------------------------------------

} // namespace swift

#endif

