// streamutil.hpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// 

#ifndef _SWIFT_STREAMUTIL_
#define _SWIFT_STREAMUTIL_

#include <iostream>

namespace swift {

// Function declaration -----------------------------------------------------

void pushback(
    std::istream & is,
    std::char_traits<char>::char_type * buf,
    std::streamsize buf_size,
    void * del_ptr
);

void pushback(
    std::istream & is,
    const std::char_traits<char>::char_type * buf,
    std::streamsize buf_size
);

std::streamsize readsome(
    std::istream & is,
    std::char_traits<char>::char_type * buf,
    std::streamsize buf_size
);

// PushbackStreamBuf class declaration --------------------------------------

class PushbackStreamBuf : public std::streambuf
{
    friend void pushback(
        std::istream &,
        std::char_traits<char>::char_type *,
        std::streamsize,
        void *
    );

    public:
    PushbackStreamBuf(
        std::istream & inputStream,
        std::char_traits<char>::char_type * buf,
        std::streamsize buf_size,
        void * del_ptr
    );

    virtual ~PushbackStreamBuf();

    protected:
    virtual std::char_traits<char>::pos_type seekoff(
        std::char_traits<char>::off_type off,
        std::ios_base::seekdir whence,
        std::ios_base::openmode which
    );

    virtual std::char_traits<char>::pos_type seekpos(
        std::char_traits<char>::pos_type pos,
        std::ios_base::openmode which
    );

    virtual std::char_traits<char>::int_type overflow(
        std::char_traits<char>::int_type c
    );
    
    std::streamsize xsputn(
        const std::char_traits<char>::char_type* buf,
        std::streamsize n
    );
    
    virtual std::char_traits<char>::int_type underflow();
    
    virtual std::streamsize xsgetn(
        std::char_traits<char>::char_type* buf,
        std::streamsize n
    );
    
    virtual std::streamsize showmanyc();

    virtual std::char_traits<char>::int_type pbackfail(
        std::char_traits<char>::int_type c = std::char_traits<char>::eof()
    );
    
    virtual int sync();

    virtual std::streambuf * setbuf(
        std::char_traits<char>::char_type* buf,
        std::streamsize buf_size
    );

    private:
    void x_FillBuffer(std::streamsize max_size);
    void x_DropBuffer();
 
    std::istream & m_Is;      //!< I/O stream this streambuf is attached to
    std::streambuf * m_Sb;      //!< original streambuf
    std::char_traits<char>::char_type * m_Buf;     //!< == 0 when the buffer has been emptied
    std::streamsize m_BufSize;
    void * m_DelPtr;
 
#ifdef HAVE_GOOD_IOS_CALLBACKS
     int m_Index;
     static void x_Callback(std::ios_base::event, std::ios_base&, int);
#endif //!<HAVE_GOOD_IOS_CALLBACKS
 
    static const std::streamsize k_MinBufSize;
};  

// --------------------------------------------------------------------------

} // namespace swift

#endif
