// swiftdatabase.hpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
// 

#ifndef _SWIFT_SWIFTDATABASE_
#define _SWIFT_SWIFTDATABASE_

#include "sequencedatabase.hpp"
#include <boost/shared_ptr.hpp>
#include <boost/utility.hpp>
#include <string>

namespace swift {

namespace detail {
    struct SwiftDatabaseImpl;
};

// SwiftDatabase class declaration ------------------------------------------

class SwiftDatabase : public SequenceDatabase, public boost::noncopyable
{
    public:
    typedef SequenceDatabase::size_type size_type;

    SwiftDatabase();
    SwiftDatabase(std::string const & name);

    std::string text(size_type n) const;

    Sequence * sequence(
        size_type n,
        Sequence::size_type begin = 0,
        Sequence::size_type end = Sequence::npos
    ) const;

    size_type offset(size_type n) const;
    Sequence::size_type size(size_type n) const;

    size_type size() const;
    size_type count() const;

    size_type find(size_type position, size_type index) const;
    size_type find(std::string const & key) const;

    std::string const & name() const;

    void open(std::string const & name);
    bool isOpen() const;
    void close();

    private:
    typedef detail::SwiftDatabaseImpl ImplType;
    boost::shared_ptr<ImplType> pimpl_;
};

// --------------------------------------------------------------------------

} // namespace swift

#endif
    
