// thousands.cpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the 
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//   

#include "thousands.hpp"
#include <algorithm>
#include <cmath>


#include <iostream>

namespace swift
{


/*!
    Returns the size of the string containing the integer \a value with
    thousands separators.
*/
size_t Thousands::size(unsigned int value)
{
    unsigned int digits(
        (unsigned int)(std::ceil(std::log10(static_cast<double>(value))))
    );
    unsigned int separators(
        (unsigned int)(std::ceil(static_cast<double>(digits) / 3) - 1)
    );
    return digits + separators;
}


/*!
    Returns a string containing the integer \a value with thousands
    separators.
*/
std::string Thousands::string(unsigned int value, char separator)
{
    std::string s;
    s.reserve(size(value));
    unsigned int c(0);

    do {
        s.push_back((char)(value % 10) + '0');
        value /= 10;
        if (value && !(++c % 3))
            s.push_back(separator);
    } while (value);

    std::reverse(s.begin(), s.end());

    return s;
}


} // namespace swift
