# FindBOOST.cmake
# Written by Kim Roland Rasmussen <krr@elreki.net>
#
# Macros for using Boost C++ Libraries with CMake.
#
# This modules defines:
#
# BOOST_INCLUDE_DIR   where to find boost includes
# BOOST_FOUND         if false, don't try to use Boost.
# BOOST_USE           macro for declaring use of Boost libraries. E.g.
#                     BOOST_USE(filesystem) defines BOOST_filesystem_LIBRARY.
#
# Permission to use this code is freely granted, provided that this statement
# is retained.

SET(BOOST_LIB_SUFFIX "gcc")

MACRO(BOOST_USE name)
    MESSAGE(STATUS "Using Boost C++ Library: ${name}")
    FIND_LIBRARY(BOOST_${name}_LIBRARY
        NAMES   boost_${name}-gcc41-1_34
              boost_${name}
              boost_${name}-${BOOST_LIB_SUFFIX}
              boost_${name}-${BOOST_LIB_SUFFIX}-${BOOST_VER}
        PATHS ${BOOST_ROOT_DIR}/lib
              ${BOOST_ROOT_DIR}/stage/lib
              /usr/local/lib
              /vol/local/lib
              /usr/lib
              /vol/gi-sw/lib
    )
    message(STATUS " found: ${BOOST_${name}_LIBRARY}")
    MARK_AS_ADVANCED(BOOST_${name}_LIBRARY)
ENDMACRO(BOOST_USE)

MESSAGE(STATUS "Looking for Boost C++ Libraries")

FIND_PATH(MY_BOOST_INCLUDE_DIR boost/config.hpp
    /usr/include
    /vol/local/include
    /usr/local/include
    /vol/gi-sw/include
)
    MARK_AS_ADVANCED(MY_BOOST_INCLUDE_DIR)


SET(BOOST_FOUND "NO")

IF(MY_BOOST_INCLUDE_DIR)
    EXEC_PROGRAM(
        ${CMAKE_CXX_COMPILER}
        ARGS -I${MY_BOOST_INCLUDE_DIR} -E \"${CMAKE_SOURCE_DIR}/config/CheckBoostVersion.cpp\"
        OUTPUT_VARIABLE CMAKE_COMPILER_OUTPUT
        RETURN_VALUE CMAKE_COMPILER_RETURN
    )

    STRING(REGEX MATCH "version=\"[0-9_]+\"" OUTP "${CMAKE_COMPILER_OUTPUT}")
    STRING(REGEX MATCH "[0-9_]+" BOOST_VERSION "${OUTP}")
    SET(BOOST_INCLUDE_DIR ${MY_BOOST_INCLUDE_DIR})
    SET(BOOST_FOUND "YES")
    MARK_AS_ADVANCED(BOOST_INCLUDE_DIR)
ENDIF(MY_BOOST_INCLUDE_DIR)

IF(BOOST_FOUND)
    MESSAGE(STATUS "Looking for Boost C++ Libraries - found (version ${BOOST_VERSION})")
ELSE(BOOST_FOUND)
    MESSAGE(STATUS "Looking for Boost C++ Libraries - not found")
ENDIF(BOOST_FOUND)

