// swift.cpp
//
// Copyright (c) 2004-2006, Kim Roland Rasmussen <krr@elreki.net>
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation files
// (the "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//
//
// Additions / changes by Peter Husemann
//

#include "createcommand.hpp"
#include "helpcommand.hpp"
#include "indexcommand.hpp"
#include "infocommand.hpp"
#include "querycommand.hpp"
#include "versioncommand.hpp"
#include <boost/scoped_array.hpp>
#include <boost/scoped_ptr.hpp>
#include <iostream>
#include <string>


int main(int argc, char ** argv)
{
    using namespace swift;


    // Get the name of the executable
    std::string executable;
    std::string tmpname = argv[0];
    int basepos = tmpname.rfind("/");

    if (basepos != std::string::npos) {
      executable = tmpname.substr(basepos + 1 ,tmpname.size());
    } else {
      executable = "swift";
    }



    if (argc < 2) {
      std::cout << "Type '"<< executable <<" help' for usage information." << std::endl;
        return 0;
    }

    // Setup command
    boost::scoped_ptr<SwiftCommand> command;
    command->SwiftExecutable=executable;
    std::string argument(argv[1]);

    if (argument == "query") {
        command.reset(new QueryCommand);
    } else if (argument == "create") {
        command.reset(new CreateCommand);
    } else if (argument == "index") {
        command.reset(new IndexCommand);
    } else if (argument == "info") {
        command.reset(new InfoCommand);
    } else if (argument == "help" || argument == "--help" ||
               argument == "-h" || argument == "-?") {
        command.reset(new HelpCommand);
    } else if (argument == "version" || argument == "--version") {
        command.reset(new VersionCommand);
    } else {
        std::cout << "Unknown command '" << argument << "'\n"
            "Type '"<< executable <<" help' for usage information." << std::endl;
        return 0;
    }

    // Prepare command arguments
    boost::scoped_array<char *> commandArguments(new char * [argc - 1]);
    // store the name of the executable in commandArguments[0]
    // commandArguments[0] = const_cast<char*>(executable.c_str()); // was argv[0]
    for (size_t i(2); i < argc; ++i)
        commandArguments[i - 1] = argv[i];
    --argc;

    // Go!
    command->execute(argc, commandArguments.get());

    return 0;
}

