
#include "fastastreamiterator.hpp"
#include <iostream>
#include <fstream>
#include <string>
#include <vector>

using namespace swift;

std::string dataPath;

void getStream(std::ifstream & stream, char const * testFile, int testnum)
{
    std::string fileName(dataPath + "/" + testFile);
    stream.open(fileName.c_str());
    if (!stream.is_open())
        throw testnum;
}

void test1()
{
    static int testnum = 1;

    std::ifstream stream;
    getStream(stream, "fa1.fa", testnum);

    FastaStreamIterator s(stream);
    FastaStreamIterator end;

    int seqCount(0);
    while (s != end) {
        ++s;
        ++seqCount;
    }

    if (seqCount != 3)
        throw testnum;
}

void test2()
{
    static int testnum = 2;

    std::vector<int> expected;
    expected.push_back(12*12*12);
    expected.push_back(4);
    expected.push_back(0);
    
    std::ifstream stream;
    getStream(stream, "fa2.fa", testnum);

    FastaStreamIterator s(stream);
    FastaStreamIterator end;

    int seqCount(0);
    while (s != end) {
        if (seqCount >= expected.size()) {
            std::cout << "More sequences reported than expected." << std::endl;
            throw testnum;
        }

        if ((*s).sequence.size() != expected[seqCount]) {
            std::cout << "Wrong sequence length." << std::endl;
            throw testnum;
        }

        std::cout << "{" << (*s).defline << "}" << std::endl;
        std::cout << (*s).sequence << std::endl;

        ++s;
        ++seqCount;
    }
}

void test3()
{
    static int testnum = 3;

    std::ifstream stream;
    getStream(stream, "fa3.fa", testnum);

    FastaStreamIterator s(stream);
    FastaStreamIterator end;

    int seqCount(0);
    while (s != end) {
        std::cout << "{" << (*s).defline << "}" << std::endl;
        std::cout << (*s).sequence << std::endl;
        ++s;
        ++seqCount;
    }

    if (seqCount != 1)
        throw testnum;
}

int main(int argc, char ** argv)
{
    if (argc < 2) {
        std::cout << "Missing directory argument." << std::endl;
        return -1;
    } else {
        dataPath = argv[1];
    }

    try {
        test1();
        test2();
        test3();
    } catch (int const & err) {
        std::cout << "Test #" << err << " failed." << std::endl;
        return -1;
    }

    return 0;
    
}
