
#include "bitstring.hpp"
#include "gramview.hpp"
#include <iostream>
#include <string>

// FIXME 1145621539
// The test only excercises forward operation of the GramView's iterators.
// Should also test backward operation.

using namespace swift;

typedef GramView<unsigned char const *> View;

std::string str("acgtACGTNNNNnnnnacgtACGT");
unsigned char * seq = 0;

void setupSeq(std::string const & str, unsigned char * seq)
{
    for (int i(0); i < str.size(); ++i) {
        unsigned char s;
        switch (str[i]) {
            case 'a': s = 0; break;
            case 'c': s = 1; break;
            case 'g': s = 2; break;
            case 't': s = 3; break;
            case 'A': s = 4; break;
            case 'C': s = 5; break;
            case 'G': s = 6; break;
            case 'T': s = 7; break;
            case 'n': s = 12; break;
            case 'N': s = 8; break;
            default: s = 8; break;
        }
        seq[i] = s;
    }
    seq[str.size()] = '\0';
}

void test1()
{
    static int testnum = 1;

    Gram::Value exp[] = {
        6939, 27756, 45488, 50880, 6912, 27648, 45056, 49152,
        0, 0, 1, 6, 27, 108, 433, 1734, 6939
    };

    unsigned int gramSize = 8;
    MaskSensitivity ms = MaskSensitive;

    View gv(seq, seq + str.size(), gramSize, ms);
    if (gv.gramSize() != gramSize || gv.maskSensitivity() != ms)
        throw testnum;

    View::const_iterator i(gv.begin());
    View::const_iterator j(gv.end());
    unsigned int indx(0);
    while (i != j) {
        if ((*i).isValid()) {
            if (exp[indx] != (*i).value())
                throw testnum;
        }
        std::cout << indx << "   " << ((*i).isValid() ? ' ' : '!') << "   "
                  << (*i).value() << "   " << exp[indx] << '\n';
        ++i;
        ++indx;
    }
    std::cout << std::endl;
}

void test2()
{
    static int testnum = 2;

    Gram::Value exp[] = {
        6939, 27756, 45488, 50880, 6912, 27648, 45056, 49152,
        0, 0, 1, 6, 27, 108, 433, 1734, 6939
    };

    View gv(seq, seq + str.size(), 8, MaskInsensitive);
    View::const_iterator i(gv.begin());
    View::const_iterator j(gv.end());
    unsigned int indx(0);
    while (i != j) {
        if ((*i).isValid()) {
            if (exp[indx] != (*i).value())
                throw testnum;
        }
        std::cout << indx << "   " << ((*i).isValid() ? ' ' : '!') << "   "
                  << (*i).value() << "   " << exp[indx] << '\n';
        ++i;
        ++indx;
    }
    std::cout << std::endl;
}

void test3()
{
    static int testnum = 3;

    Gram::Value exp[] = {
        6939, 27756, 45488, 50880, 6912, 27648, 45056, 49152,
        0, 0, 1, 6, 27, 108, 433, 1734, 6939
    };

    unsigned int gramSize = 8;

    for (int len(0); len < gramSize; ++len) {
        View gv(seq, seq + len, gramSize, MaskInsensitive);
        View::const_iterator i(gv.begin());
        View::const_iterator j(gv.end());

        if (gv.size() > 0) {
            std::cout << "size() == " << gv.size() << ", expected zero.\n";
            throw testnum;
        }

        if (i != j) {
            std::cout << "i != j" << std::endl;
            throw testnum;
        }

        if ((*i).isValid()) {
            std::cout << bitstring((*i).value()) << std::endl;
            throw testnum;
        }
    }
}

int main(int argc, char ** argv)
{
    seq = new unsigned char[str.size() + 1]; 
    setupSeq(str, seq);
    
    try {
        test1();
        test2();
        test3();
    } catch (int const & err) {
        std::cout << "Test #" << err << " failed." << std::endl;
        return -1;
    }

    return 0;
    
}
